#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.hybrid_shape_sweep import HybridShapeSweep
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.knowledge_interfaces.length import Length


class HybridShapeSweepCircle(HybridShapeSweep):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         CATGSMIDLItf.HybridShapeSweep
                |                             HybridShapeSweepCircle
                | 
                | Represents the hybrid shape sweep circle feature object.
                | Role: To access the data of the hybrid shape sweep circle feature
                | object.
                | 
                | LICENSING INFORMATION: Creation of volume result requires GSO
                | License
                | if GSO License is not granted , setting of Volume context has not
                | effect
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_sweep_circle = com_object

    @property
    def canonical_detection(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property CanonicalDetection() As long
                | 
                |     Returns or sets whether canonical surfaces of the swept surface are
                |     detected.
                |     Legal values:
                |     0 No detection of canonical surface is performed.
                |     2 Detection of canonical surfaces is performed.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.CanonicalDetection

    @canonical_detection.setter
    def canonical_detection(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.CanonicalDetection = value

    @property
    def choice_no(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ChoiceNo() As long
                | 
                |     Returns or sets the choice number, which corresponds to each solution of a
                |     given circular sweep case.
                |     For example: a circular sweep with two guide curves and a radius leads to
                |     four possible solutions.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.ChoiceNo

    @choice_no.setter
    def choice_no(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.ChoiceNo = value

    @property
    def context(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Context() As long
                | 
                |     Returns or sets the context on Sweep feature.
                | 
                |         0 This option creates Swept surface.
                |         1 This option creates Swept volume.
                | 
                | 
                |     Note: Setting volume result requires GSO License.
                | 
                |     Example:
                |         This example retrieves in oContext the context for the Sweep hybrid
                |         shape feature.
                | 
                |          Dim oContext
                |          Set oContext = Sweep.Context

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.Context

    @context.setter
    def context(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.Context = value

    @property
    def first_angle_law(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FirstAngleLaw() As Reference
                | 
                |     Returns or sets the first angle law useful in some circular sweep types.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.FirstAngleLaw)

    @first_angle_law.setter
    def first_angle_law(self, reference_law: Reference):
        """
        :param Reference reference_law:
        """

        self.hybrid_shape_sweep_circle.FirstAngleLaw = reference_law.com_object

    @property
    def first_angle_law_inversion(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FirstAngleLawInversion() As long
                | 
                |     Returns or sets the first angle law inversion information.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.FirstAngleLawInversion

    @first_angle_law_inversion.setter
    def first_angle_law_inversion(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.FirstAngleLawInversion = value

    @property
    def first_guide_crv(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FirstGuideCrv() As Reference
                | 
                |     Returns or sets the sweep operation first guide curve.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.FirstGuideCrv)

    @first_guide_crv.setter
    def first_guide_crv(self, reference_curve: Reference):
        """
        :param Reference reference_curve:
        """

        self.hybrid_shape_sweep_circle.FirstGuideCrv = reference_curve.com_object

    @property
    def guide_deviation(self) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property GuideDeviation() As Length (Read Only)
                | 
                |     Returns the deviation value (length) from guide curves allowed during a
                |     sweeping operation in order to smooth it.

        :rtype: Length
        """

        return Length(self.hybrid_shape_sweep_circle.GuideDeviation)

    @property
    def guide_deviation_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property GuideDeviationActivity() As boolean
                | 
                |     Returns or sets information whether a deviation from guide curves is
                |     allowed or not.
                |     Gives the information on performing smoothing during sweeping
                |     operation.
                |     TRUE if a deviation from guide curves is allowed, or FALSE otherwise (FALSE
                |     if not specified).

        :rtype: bool
        """

        return self.hybrid_shape_sweep_circle.GuideDeviationActivity

    @guide_deviation_activity.setter
    def guide_deviation_activity(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_sweep_circle.GuideDeviationActivity = value

    @property
    def mode(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Mode() As long
                | 
                |     Returns or sets the circular sweep mode.
                |     Legal mode values are:
                |     0 Undefined circular profile swept surface
                |     (CATGSMCircularSweep_None)
                |     2 Circular profile swept surface defined by three guide curves (4
                |     solutions) (CATGSMCircularSweep_ThreeGuides)
                |     3 Circular profile swept surface defined by a center curve and a reference
                |     curve (for angles and radius)
                |     (CATGSMCircularSweep_TwoGuidesAndRadius)
                |     5 Circular profile swept surface defined by a center curve and a reference
                |     curve (for angles and radius)
                |     (CATGSMCircularSweep_CenterAndAngleCurve)
                |     6 Circular profile swept surface defined by a center curve and a radius
                |     (CATGSMCircularSweep_CenterAndRadius)
                |     7 Circular profile swept surface defined by two guide curves with a
                |     tangency condition on the second one (with reference surface)
                |     (CATGSMCircularSweep_TwoGuidesAndTangency)
                |     8 Circular profile swept surface defined by a guide curve, a radius and a
                |     tangency surface
                |     (CATGSMCircularSweep_GuideAndTangencyAndRadius)

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.Mode

    @mode.setter
    def mode(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.Mode = value

    @property
    def radius_law(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property RadiusLaw() As Reference
                | 
                |     Returns or sets the radius law feature.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.RadiusLaw)

    @radius_law.setter
    def radius_law(self, reference_law: Reference):
        """
        :param Reference reference_law:
        """

        self.hybrid_shape_sweep_circle.RadiusLaw = reference_law.com_object

    @property
    def radius_law_inversion(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property RadiusLawInversion() As long
                | 
                |     Returns or sets the radius law inversion information.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.RadiusLawInversion

    @radius_law_inversion.setter
    def radius_law_inversion(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.RadiusLawInversion = value

    @property
    def radius_law_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property RadiusLawType() As long
                | 
                |     Returns or sets the radius law type.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.RadiusLawType

    @radius_law_type.setter
    def radius_law_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.RadiusLawType = value

    @property
    def reference(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Reference() As Reference
                | 
                |     Returns or sets the reference (functional curve or guide surface).

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.Reference)

    @reference.setter
    def reference(self, reference: Reference):
        """
        :param Reference reference:
        """

        self.hybrid_shape_sweep_circle.Reference = reference.com_object

    @property
    def second_angle_law(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SecondAngleLaw() As Reference
                | 
                |     Returns or sets the second angle law useful in some circular sweep types.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.SecondAngleLaw)

    @second_angle_law.setter
    def second_angle_law(self, reference_law: Reference):
        """
        :param Reference reference_law:
        """

        self.hybrid_shape_sweep_circle.SecondAngleLaw = reference_law.com_object

    @property
    def second_angle_law_inversion(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SecondAngleLawInversion() As long
                | 
                |     Returns or sets the second angle law inversion information.

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.SecondAngleLawInversion

    @second_angle_law_inversion.setter
    def second_angle_law_inversion(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.SecondAngleLawInversion = value

    @property
    def second_guide_crv(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SecondGuideCrv() As Reference
                | 
                |     Returns or sets the sweep operation second guide curve.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.SecondGuideCrv)

    @second_guide_crv.setter
    def second_guide_crv(self, reference_curve: Reference):
        """
        :param Reference reference_curve:
        """

        self.hybrid_shape_sweep_circle.SecondGuideCrv = reference_curve.com_object

    @property
    def smooth_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothActivity() As boolean
                | 
                |     Returns or sets information whether a sweeping operation is smoothed or
                |     not.
                |     TRUE if the sweeping operation is smoothed, or FALSE otherwise (FALSE if
                |     not specified).

        :rtype: bool
        """

        return self.hybrid_shape_sweep_circle.SmoothActivity

    @smooth_activity.setter
    def smooth_activity(self, value: bool):
        """
        :param bool value:
        """

        self.hybrid_shape_sweep_circle.SmoothActivity = value

    @property
    def smooth_angle_threshold(self) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property SmoothAngleThreshold() As Angle (Read Only)
                | 
                |     Returns the angular threshold.

        :rtype: Angle
        """

        return Angle(self.hybrid_shape_sweep_circle.SmoothAngleThreshold)

    @property
    def spine(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Spine() As Reference
                | 
                |     Returns or sets the sweep operation spine (optional).

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.Spine)

    @spine.setter
    def spine(self, reference_spine: Reference):
        """
        :param Reference reference_spine:
        """

        self.hybrid_shape_sweep_circle.Spine = reference_spine.com_object

    @property
    def third_guide_crv(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property ThirdGuideCrv() As Reference
                | 
                |     Returns or sets the sweep operation third guide curve.

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_sweep_circle.ThirdGuideCrv)

    @third_guide_crv.setter
    def third_guide_crv(self, reference_curve: Reference):
        """
        :param Reference reference_curve:
        """

        self.hybrid_shape_sweep_circle.ThirdGuideCrv = reference_curve.com_object

    @property
    def trim_option(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property TrimOption() As long
                | 
                |     Returns or sets the trim option status.
                |     The trim option status legal values are:
                |     0 No trim computed or undefined
                |     (CATGSMSweepTrimMode_None)
                |     1 Trim computed (CATGSMSweepTrimMode_On)

        :rtype: int
        """

        return self.hybrid_shape_sweep_circle.TrimOption

    @trim_option.setter
    def trim_option(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_sweep_circle.TrimOption = value

    def get_angle(self, i_i: int) -> Angle:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetAngle(long iI) As Angle
                | 
                |     Returns the angle values useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         iI
                |             The angle value index 
                | 
                |     Returns:
                |         The angle value

        :param int i_i:
        :rtype: Angle
        """
        return Angle(self.hybrid_shape_sweep_circle.GetAngle(i_i))

    def get_angle_law_types(self, o_first_type: int, o_second_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetAngleLawTypes(long oFirstType,
                | long oSecondType)
                | 
                |     Retrieves angle law types.
                | 
                |     Parameters:
                | 
                |         oFirstType
                |             The first type of law (from CATGSMBasicLawType
                |             enumeration).
                |             0 Undefined law type (CATGSMBasicLawType_None)
                |             1 Constant law type (CATGSMBasicLawType_Constant)
                |             2 Linear law type (CATGSMBasicLawType_Linear)
                |             3 S law type (CATGSMBasicLawType_SType)
                |             4 Law specified by a GSD law feature
                |             (CATGSMBasicLawType_Advanced)
                |         oSecondType
                |             The second type of law (from CATGSMBasicLawType
                |             enumeration).
                |             Same legal values as oFirstType

        :param int o_first_type:
        :param int o_second_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetAngleLawTypes(o_first_type, o_second_type)

    def get_first_angle_law(self, o_elem1: Angle, o_elem2: Angle, ol_law_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetFirstAngleLaw(Angle oElem1,
                | Angle oElem2,
                | long olLawType)
                | 
                |     Retrieves the first angle law useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         oElem1
                |             The angle law start value 
                |         oElem2
                |             The angle law end value 
                |         olLawType
                |             The angle law type

        :param Angle o_elem1:
        :param Angle o_elem2:
        :param int ol_law_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetFirstAngleLaw(o_elem1.com_object, o_elem2.com_object, ol_law_type)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_first_angle_law'
        # # vba_code = """
        # # Public Function get_first_angle_law(hybrid_shape_sweep_circle)
        # #     Dim oElem1 (2)
        # #     hybrid_shape_sweep_circle.GetFirstAngleLaw oElem1
        # #     get_first_angle_law = oElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_longitudinal_relimiters(self, op_ia_elem1: Reference, op_ia_elem2: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetLongitudinalRelimiters(Reference opIAElem1,
                | Reference opIAElem2)
                | 
                |     Deprecated:
                |         V5R16 CATHybridShapeSweepCircle#GetRelimiters Retrieves the elements
                |         relimiting the spine (or the default spine). 
                |     Parameters:
                | 
                |         opIAElem1
                |             The first relimiting feature (plane or point) 
                |         opIAElem2
                |             The second relimiting feature (plane or point)

        :param Reference op_ia_elem1:
        :param Reference op_ia_elem2:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetLongitudinalRelimiters(op_ia_elem1.com_object, op_ia_elem2.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_longitudinal_relimiters'
        # # vba_code = """
        # # Public Function get_longitudinal_relimiters(hybrid_shape_sweep_circle)
        # #     Dim opIAElem1 (2)
        # #     hybrid_shape_sweep_circle.GetLongitudinalRelimiters opIAElem1
        # #     get_longitudinal_relimiters = opIAElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_nb_angle(self, o_ang: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetNbAngle(long oAng)
                | 
                |     Retrieves the number of angles.
                | 
                |     Parameters:
                | 
                |         oAng
                |             The number of angles

        :param int o_ang:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetNbAngle(o_ang)

    def get_nb_guide(self, o_num: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetNbGuide(long oNum)
                | 
                |     Retrieves the number of guide curves.
                | 
                |     Parameters:
                | 
                |         oNum
                |             The number of guide curves

        :param int o_num:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetNbGuide(o_num)

    def get_nb_radius(self, o_rad: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetNbRadius(long oRad)
                | 
                |     Retrieves the number of radii.
                | 
                |     Parameters:
                | 
                |         oRad
                |             The number of radii

        :param int o_rad:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetNbRadius(o_rad)

    def get_radius(self, i_i: int) -> Length:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func GetRadius(long iI) As Length
                | 
                |     Returns the radius value useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         iI
                |             The radius value index (1: start value, 2: end value)
                |             
                | 
                |     Returns:
                |         The radius value

        :param int i_i:
        :rtype: Length
        """
        return Length(self.hybrid_shape_sweep_circle.GetRadius(i_i))

    def get_relimiters(self, op_ia_elem1: Reference, op_orient1: int, op_ia_elem2: Reference, op_orient2: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetRelimiters(Reference opIAElem1,
                | long opOrient1,
                | Reference opIAElem2,
                | long opOrient2)
                | 
                |     Retrieves the elements relimiting the spine (or the default
                |     spine).
                | 
                |     Parameters:
                | 
                |         opIAElem1
                |             The first relimiting feature (plane or point) 
                |         opOrient1
                |             Split direction for the first relimitation
                |             0 means that the beginning of the spine (considering its
                |             orientation) is removed, 1 means that the end of the spine is removed
                |             
                |         opIAElem2
                |             The second relimiting feature (plane or point) 
                |         opOrient2
                |             Split direction for the second relimitation

        :param Reference op_ia_elem1:
        :param int op_orient1:
        :param Reference op_ia_elem2:
        :param int op_orient2:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetRelimiters(op_ia_elem1.com_object, op_orient1, op_ia_elem2.com_object,
                                                            op_orient2)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_relimiters'
        # # vba_code = """
        # # Public Function get_relimiters(hybrid_shape_sweep_circle)
        # #     Dim opIAElem1 (2)
        # #     hybrid_shape_sweep_circle.GetRelimiters opIAElem1
        # #     get_relimiters = opIAElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_second_angle_law(self, o_elem1: Angle, o_elem2: Angle, ol_law_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetSecondAngleLaw(Angle oElem1,
                | Angle oElem2,
                | long olLawType)
                | 
                |     Retrieves the second angle law useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         oElem1
                |             The angle law start value 
                |         oElem2
                |             The angle law end value 
                |         olLawType
                |             The angle law type

        :param Angle o_elem1:
        :param Angle o_elem2:
        :param int ol_law_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetSecondAngleLaw(o_elem1.com_object, o_elem2.com_object, ol_law_type)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_second_angle_law'
        # # vba_code = """
        # # Public Function get_second_angle_law(hybrid_shape_sweep_circle)
        # #     Dim oElem1 (2)
        # #     hybrid_shape_sweep_circle.GetSecondAngleLaw oElem1
        # #     get_second_angle_law = oElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_tangency_choice_no(self, o_no: int, o_shell_ori: int, o_guide_ori: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub GetTangencyChoiceNo(long oNo,
                | long oShellOri,
                | long oGuideOri)
                | 
                |     Retrieves a sequence which identifies a solution among all possibilities of
                |     a circular profile sweep tangent to a surface. (case
                |     CATGSMCircularSweep_GuideAndTangencyAndRadius).
                | 
                |     Parameters:
                | 
                |         oNo
                |             Given the orientations, solution number in a distance ordered list.
                |             
                |         oShellOri
                |             This orientation allows to compute just the results that are
                |             tangent to a specific side of the shell. It can take three
                |             values:
                |             +1 The result is on the normal side of the shell
                |             -1 The result is on the side of the shell opposite to the
                |             normal
                |             0 No orientation is specified
                |         oGuideOri
                |             This orientation allows to compute just the results that are on the
                |             "left" or the "right" side of the shell, when looking in the guide direction.
                |             It can take three values:
                |             +1 The result is on the "left" side
                |             -1 The result is on the "right" side
                |             0 No orientation is specified

        :param int o_no:
        :param int o_shell_ori:
        :param int o_guide_ori:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.GetTangencyChoiceNo(o_no, o_shell_ori, o_guide_ori)

    def remove_angle(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveAngle()
                | 
                |     Removes an angle.

        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.RemoveAngle()

    def remove_guide(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveGuide()
                | 
                |     Removes a guide curve.

        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.RemoveGuide()

    def remove_radius(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub RemoveRadius()
                | 
                |     Removes a radius.

        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.RemoveRadius()

    def set_angle(self, i_i: int, i_elem: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetAngle(long iI,
                | double iElem)
                | 
                |     Sets the angle values useful in some circular sweep types.
                | 
                |     Parameters:
                | 
                |         iI
                |             The angle value index 
                |         iElem
                |             The angle value

        :param int i_i:
        :param float i_elem:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetAngle(i_i, i_elem)

    def set_angle_law_types(self, i_first_type: int, i_second_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetAngleLawTypes(long iFirstType,
                | long iSecondType)
                | 
                |     Sets angle law types.
                | 
                |     Parameters:
                | 
                |         iFirstType
                |             The first type of law (from CATGSMBasicLawType
                |             enumeration).
                |             Legal values:
                |             0 Undefined law type (CATGSMBasicLawType_None)
                |             1 Constant law type (CATGSMBasicLawType_Constant)
                |             2 Linear law type (CATGSMBasicLawType_Linear)
                |             3 S law type (CATGSMBasicLawType_SType)
                |             4 Law specified by a GSD law feature
                |             (CATGSMBasicLawType_Advanced)
                |         iSecondType
                |             The second type of law (from CATGSMBasicLawType
                |             enumeration).
                |             Same legal values as iFirstType

        :param int i_first_type:
        :param int i_second_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetAngleLawTypes(i_first_type, i_second_type)

    def set_first_angle_law(self, i_elem1: float, i_elem2: float, il_law_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetFirstAngleLaw(double iElem1,
                | double iElem2,
                | long ilLawType)
                | 
                |     Sets the first angle law useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         iElem1
                |             The angle law start value 
                |         iElem2
                |             The angle law end value 
                |         ilLawType
                |             The angle law type

        :param float i_elem1:
        :param float i_elem2:
        :param int il_law_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetFirstAngleLaw(i_elem1, i_elem2, il_law_type)

    def set_guide_deviation(self, i_length: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetGuideDeviation(double iLength)
                | 
                |     Sets the deviation value (length) from guide curves allowed during sweeping
                |     operation in order to smooth it.
                | 
                |     Parameters:
                | 
                |         iLength
                |             The deviation value

        :param float i_length:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetGuideDeviation(i_length)

    def set_longitudinal_relimiters(self, ip_ia_elem1: Reference, ip_ia_elem2: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetLongitudinalRelimiters(Reference ipIAElem1,
                | Reference ipIAElem2)
                | 
                |     Deprecated:
                |         V5R16 CATHybridShapeSweepCircle#SetRelimiters Sets the elements
                |         relimiting the spine (or the default spine). 
                |     Parameters:
                | 
                |         ipIAElem1
                |             The first relimiting feature (plane or point) 
                |         ipIAElem2
                |             The second relimiting feature (plane or point)

        :param Reference ip_ia_elem1:
        :param Reference ip_ia_elem2:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetLongitudinalRelimiters(ip_ia_elem1.com_object, ip_ia_elem2.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_longitudinal_relimiters'
        # # vba_code = """
        # # Public Function set_longitudinal_relimiters(hybrid_shape_sweep_circle)
        # #     Dim ipIAElem1 (2)
        # #     hybrid_shape_sweep_circle.SetLongitudinalRelimiters ipIAElem1
        # #     set_longitudinal_relimiters = ipIAElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_radius(self, i_i: int, i_radius: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetRadius(long iI,
                | double iRadius)
                | 
                |     Sets the radius value useful in some circular sweep types.
                | 
                |     Parameters:
                | 
                |         iI
                |             The radius value index (1: start value, 2: end value)
                |             
                |         iRadius
                |             The radius value

        :param int i_i:
        :param float i_radius:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetRadius(i_i, i_radius)

    def set_relimiters(self, ip_ia_elem1: Reference, ip_orient1: int, ip_ia_elem2: Reference, ip_orient2: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetRelimiters(Reference ipIAElem1,
                | long ipOrient1,
                | Reference ipIAElem2,
                | long ipOrient2)
                | 
                |     Sets the elements relimiting the spine (or the default
                |     spine).
                | 
                |     Parameters:
                | 
                |         ipIAElem1
                |             The first relimiting feature (plane or point) 
                |         ipOrient1
                |             Split direction for the first relimitation
                |             0 means that the beginning of the spine (considering its
                |             orientation) is removed, 1 means that the end of the spine is removed
                |             
                |         ipIAElem2
                |             The second relimiting feature (plane or point) 
                |         ipOrient2
                |             Split direction for the second relimitation

        :param Reference ip_ia_elem1:
        :param int ip_orient1:
        :param Reference ip_ia_elem2:
        :param int ip_orient2:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetRelimiters(ip_ia_elem1.com_object, ip_orient1, ip_ia_elem2.com_object,
                                                            ip_orient2)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_relimiters'
        # # vba_code = """
        # # Public Function set_relimiters(hybrid_shape_sweep_circle)
        # #     Dim ipIAElem1 (2)
        # #     hybrid_shape_sweep_circle.SetRelimiters ipIAElem1
        # #     set_relimiters = ipIAElem1
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_second_angle_law(self, i_elem1: float, i_elem2: float, il_law_type: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetSecondAngleLaw(double iElem1,
                | double iElem2,
                | long ilLawType)
                | 
                |     Sets the second angle law useful in some circular sweep
                |     types.
                | 
                |     Parameters:
                | 
                |         iElem1
                |             The angle law start value 
                |         iElem2
                |             The angle law end value 
                |         ilLawType
                |             Tha angle law type

        :param float i_elem1:
        :param float i_elem2:
        :param int il_law_type:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetSecondAngleLaw(i_elem1, i_elem2, il_law_type)

    def set_smooth_angle_threshold(self, i_angle: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetSmoothAngleThreshold(double iAngle)
                | 
                |     Sets the angular threshold.
                | 
                |     Parameters:
                | 
                |         iAngle
                |             The angular threshold

        :param float i_angle:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetSmoothAngleThreshold(i_angle)

    def set_tangency_choice_no(self, i_shell_ori: int, i_guide_ori: int, i_no: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub SetTangencyChoiceNo(long iShellOri,
                | long iGuideOri,
                | long iNo)
                | 
                |     Sets a sequence which identifies a solutionamong all possibilities of a
                |     circular profile sweep tangent to a surface.
                | 
                |     Parameters:
                | 
                |         iNo
                |             Given the orientations, solution number in a distance ordered list.
                |             
                |         iShellOri
                |             This orientation allows to compute just the results that are
                |             tangent to a specific side of the shell. It can take three
                |             values:
                |             +1 The result is on the normal side of the shell
                |             -1 The result is on the side of the shell opposite to the
                |             normal
                |             0 No orientation is specified
                |         iGuideOri
                |             This orientation allows to compute just the results that are on the
                |             "left" or the "right" side of the shell, when looking in the guide direction.
                |             It can take three values:
                |             +1 The result is on the "left" side
                |             -1 The result is on the "right" side
                |             0 No orientation is specified

        :param int i_shell_ori:
        :param int i_guide_ori:
        :param int i_no:
        :rtype: None
        """
        return self.hybrid_shape_sweep_circle.SetTangencyChoiceNo(i_shell_ori, i_guide_ori, i_no)

    def __repr__(self):
        return f'HybridShapeSweepCircle(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.in_interfaces.file import File
from pycatia.in_interfaces.folder import Folder
from pycatia.system_interfaces.any_object import AnyObject


class FileSystem(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     FileSystem
                | 
                | Represents the file system object.
                | Role: The file system object allows you to access and manipulate folders and
                | files. It can check the existence of, get, create, delete, or copy folders and
                | files.
                | 
                | See also:
                |     Folder, File
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.file_system = com_object

    @property
    def file_separator(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property FileSeparator() As CATBSTR (Read Only)
                | 
                |     Returns the file separator string. (usually, // on Windows and / on
                |     UNIX)
                | 
                |     Parameters:
                | 
                |         oTmpDirectory
                |             The file separator string.

        :rtype: str
        """

        return self.file_system.FileSeparator

    @property
    def path_separator(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property PathSeparator() As CATBSTR (Read Only)
                | 
                |     Returns the path separator string. (usually,; on Windows and : on UNIX)
                | 
                |     Parameters:
                | 
                |         oTmpDirectory
                |             The path separator string.

        :rtype: str
        """

        return self.file_system.PathSeparator

    @property
    def temporary_directory(self) -> Folder:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property TemporaryDirectory() As Folder (Read Only)
                | 
                |     Returns the temporary system directory. (usually, C://temp on Windows and
                |     /tmp on UNIX)
                | 
                |     Parameters:
                | 
                |         oTmpDirectory
                |             A folder which corresponds to the temporary system
                |             directory.

        :rtype: Folder
        """

        return Folder(self.file_system.TemporaryDirectory)

    def concatenate_paths(self, i_path_chunk1: str, i_path_chunk2: str) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func ConcatenatePaths(CATBSTR iPathChunk1,
                | CATBSTR iPathChunk2) As CATBSTR
                | 
                |     Concatenates two path chunks to make a new path. Either path chunk can be
                |     empty. The resulting path does not have to exist. The method automatically
                |     corrects improper path separators (/ Unix separators used on a Windows platform
                |     are automatically replaced by // and vice versa).
                | 
                |     Parameters:
                | 
                |         iPathChunk1
                |             The first path chunk (for instance "E://tmp").
                |         iPathChunk2
                |             The second path chunk (for instance "local/myfile.txt").
                |             
                |         oPath
                |             The resulting path (for instance
                |             "E://tmp//local//myfile.txt").

        :param str i_path_chunk1:
        :param str i_path_chunk2:
        :rtype: str
        """
        return self.file_system.ConcatenatePaths(i_path_chunk1, i_path_chunk2)

    def copy_file(self, i_path_source: str, i_path_destination: str, i_overwrite: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub CopyFile(CATBSTR iPathSource,
                | CATBSTR iPathDestination,
                | boolean iOverwrite)
                | 
                |     Copies a file from one location to another.
                | 
                |     Parameters:
                | 
                |         iSourcePath
                |             The full path of the source file. 
                |         iDestinationPath
                |             The full destination path where the source file is to be copied.
                |             
                |         iOverwrite
                |             Boolean value that is True if an existing file with the same name
                |             can be overwritten; False if it is not, and the copy doesn't take
                |             place.
                | 
                |             Example:
                |                 This example copies the file C:/Tests/File1 to C:/Tests/File2
                |                 from the file system object FileSys, except if a file with the name
                |                 C:/Tests/File2 already exists.
                | 
                |                  FileSys.CopyFile("C:/Tests/File1", "C:/Tests/File2",
                |                  False)

        :param str i_path_source:
        :param str i_path_destination:
        :param bool i_overwrite:
        :rtype: None
        """
        return self.file_system.CopyFile(i_path_source, i_path_destination, i_overwrite)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'copy_file'
        # # vba_code = """
        # # Public Function copy_file(file_system)
        # #     Dim iPathSource (2)
        # #     file_system.CopyFile iPathSource
        # #     copy_file = iPathSource
        # # End Function
        # # """

        # # system_service = self.application.system_service
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def copy_folder(self, i_source_path: str, i_destination_path: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub CopyFolder(CATBSTR iSourcePath,
                | CATBSTR iDestinationPath)
                | 
                |     Recursively copies a folder from one location to another. Role: The folder
                |     is copied along with its files, and all its subfolders and their own
                |     files.
                | 
                |     Parameters:
                | 
                |         iSourcePath
                |             The full path of the source folder. 
                |         iDestinationPath
                |             The full destination path where the source folder, its files, and
                |             its subfolders are to be copied.
                | 
                |             Example:
                |                 This example copies the folder "C:/Tests/Fold1" to
                |                 "C:/Tests/Fold2" of the file system object
                |                 FileSys.
                | 
                |                  FileSys.CopyFolder("C:/Tests/Fold1",
                |                  "C:/Tests/Fold2")

        :param str i_source_path:
        :param str i_destination_path:
        :rtype: None
        """
        return self.file_system.CopyFolder(i_source_path, i_destination_path)

    def create_file(self, i_path: str, i_overwrite: bool) -> File:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Func CreateFile(CATBSTR iPath,
                | boolean iOverwrite) As File
                | 
                |     Creates a file and returns the associated file object.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path where the file is to be created. 
                |         iOverwrite
                |             Boolean value that is True if an existing file with the same name
                |             can be overwritten; False if it is not, and the creation doesn't take place.
                |             
                | 
                |     Returns:
                |         The created file
                | 
                |         Example:
                |             This example creates the file C:/Tests/File1 and retrieves it in
                |             the file object FileObj from the file system object FileSys, except if a file
                |             with the name C:/Tests/File1 already exists.
                | 
                |              Dim FileObj As File
                |              Set FileObj = FileSys.CreateFile("C:/Tests/File1", False)

        :param str i_path:
        :param bool i_overwrite:
        :rtype: File
        """
        return File(self.file_system.CreateFile(i_path, i_overwrite))

    def create_folder(self, i_path: str) -> Folder:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateFolder(CATBSTR iPath) As Folder
                | 
                |     Creates a folder and returns the associated folder object.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path where the folder is to be created. 
                | 
                |     Returns:
                |         The created folder object. If the folder already exists, the method
                |         fails.
                | 
                |         Example:
                |             This example creates the folder "C:/Tests/Fold1" and retrieves it
                |             in FoldObj from the FileSystem FileSys.
                | 
                |              Dim FoldObj As Folder
                |              Set FoldObj = FileSys.CreateFolder("C:/Tests/Fold1")

        :param str i_path:
        :rtype: Folder
        """
        return Folder(self.file_system.CreateFolder(i_path))

    def delete_file(self, i_path: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub DeleteFile(CATBSTR iPath)
                | 
                |     Deletes a file.
                |     The method fails if the folder doesn't exist.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the file to delete.
                | 
                |             Example:
                |                 This example deletes the file "C:/Tests/File" from the file
                |                 system object FileSys.
                | 
                |                  FileSys.DeleteFile("C:/Tests/File1")

        :param str i_path:
        :rtype: None
        """
        return self.file_system.DeleteFile(i_path)

    def delete_folder(self, i_path: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub DeleteFolder(CATBSTR iPath)
                | 
                |     Deletes a folder.
                |     The method fails if the folder doesn't exist.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the folder to delete.
                | 
                |             Example:
                |                 This example deletes the folder "C:/Tests/Fold1" from the
                |                 FileSystem FileSys.
                | 
                |                  FileSys.DeleteFolder("C:/Tests/Fold1")

        :param str i_path:
        :rtype: None
        """
        return self.file_system.DeleteFolder(i_path)

    def file_exists(self, i_path: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func FileExists(CATBSTR iPath) As boolean
                | 
                |     Returns whether a given file exists.
                |     True if the file exists.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the file.
                | 
                |             Example:
                |                 This example retrieves in Exists whether the file
                |                 "C:/Tests/File1" exists in the file system object
                |                 FileSys.
                | 
                |                  Dim Exists As Boolean
                |                  Exists = FileSys.FileExists("C:/Tests/File1")

        :param str i_path:
        :rtype: bool
        """
        return self.file_system.FileExists(i_path)

    def folder_exists(self, i_path: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func FolderExists(CATBSTR iPath) As boolean
                | 
                |     Returns whether a given folder exists.
                |     True if the folder exists.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the folder.
                | 
                |             Example:
                |                 This example retrieves in Exists whether the folder
                |                 "C:/Tests/Fold1" exists in the file system object
                |                 FileSys.
                | 
                |                  Dim Exists As Boolean
                |                  Exists=FileSys.FolderExists("C:/Tests/Fold1")

        :param str i_path:
        :rtype: bool
        """
        return self.file_system.FolderExists(i_path)

    def get_file(self, i_path: str) -> File:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetFile(CATBSTR iPath) As File
                | 
                |     Returns a file using its full path.
                |     The method fails if the folder doesn't exist.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the file to retrieve.
                | 
                |             Example:
                |                 This example retrieves the file C:/Tests in the FileObj from
                |                 the file system object FileSys.
                | 
                |                  Dim FileObj As File
                |                  Set FileObj = FileSys.GetFile("C:/Tests")

        :param str i_path:
        :rtype: File
        """
        return File(self.file_system.GetFile(i_path))

    def get_folder(self, i_path: str) -> Folder:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetFolder(CATBSTR iPath) As Folder
                | 
                |     Returns a folder using its full path.
                | 
                |     Parameters:
                | 
                |         iPath
                |             The full path of the folder to retrieve. 
                | 
                |     Returns:
                |         The retrieved folder. If the folder doesn't exist, the method
                |         fails.
                | 
                |         Example:
                |             This example retrieves the C:Tests folder in FoldObj from the file
                |             system object FileSys.
                | 
                |              Dim FoldObj As Folder
                |              Set FoldObj = FileSys.GetFolder("C:/Tests/")

        :param str i_path:
        :rtype: Folder
        """
        return Folder(self.file_system.GetFolder(i_path))

    def __repr__(self):
        return f'FileSystem(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.part_interfaces.pattern import Pattern
from pycatia.system_interfaces.any_object import AnyObject


class UserPattern(Pattern):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Shape
                |                         PartInterfaces.TransformationShape
                |                             PartInterfaces.Pattern
                |                                 UserPattern
                | 
                | Represents the user pattern.
                | The shape is copied along user's positions.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.user_pattern = com_object

    @property
    def anchor_point(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AnchorPoint() As AnyObject
                | 
                |     Returns the anchor point of the user pattern.
                | 
                |     Example:
                |         The following example returns in anchor the anchor point of the Pattern
                |         firstPattern:
                | 
                |          Set anchor = firstPattern.AnchorPoint

        :rtype: AnyObject
        """

        return AnyObject(self.user_pattern.AnchorPoint)

    @anchor_point.setter
    def anchor_point(self, value: AnyObject):
        """
        :param AnyObject value:
        """

        self.user_pattern.AnchorPoint = value

    @property
    def feature_to_locate_positions(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property FeatureToLocatePositions() As AnyObject (Read
                | Only)
                | 
                |     Returns the collection of feature to locate instances.
                | 
                |     Example:
                |         The following example returns in list the list of feature to locate
                |         instances of the Pattern firstPattern:
                | 
                |          Set list = firstPattern.FeatureToLocatePositions

        :rtype: AnyObject
        """

        return AnyObject(self.user_pattern.FeatureToLocatePositions)

    def add_feature_to_locate_positions(self, i_feature_to_locate_positions: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384))
                | o Sub AddFeatureToLocatePositions(AnyObject
                | iFeatureToLocatePositions)
                | 
                |     Adds a new feature to locate instances.
                | 
                |     Parameters:
                | 
                |         iFeatureToLocatePositions
                |             The new object containing points of positioning 
                | 
                |     Example:
                |         The following example adds the new feature feature to locate instances
                |         of the Pattern firstPattern:
                | 
                |          call firstPattern.AddFeatureToLocatePositions(object)

        :param AnyObject i_feature_to_locate_positions:
        :rtype: None
        """
        return self.user_pattern.AddFeatureToLocatePositions(i_feature_to_locate_positions.com_object)

    def __repr__(self):
        return f'UserPattern(name="{ self.name }")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import TYPE_CHECKING

from pycatia.dmaps_interfaces.activity import Activity
from pycatia.knowledge_interfaces.parameter import Parameter
from pycatia.manufacturing_interfaces.manufacturing_machinable_area import ManufacturingMachinableArea
from pycatia.manufacturing_interfaces.manufacturing_tool import ManufacturingTool
from pycatia.manufacturing_interfaces.manufacturing_tool_assembly import ManufacturingToolAssembly

if TYPE_CHECKING:
    from pycatia.manufacturing_interfaces.manufacturing_precedences import ManufacturingPrecedences


class ManufacturingActivity(Activity):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     DMAPSInterfaces.Activity
                |                         ManufacturingActivity
                | 
                | ManufacturingActivity defines a set of methods and mainly adds some specific
                | methods and services in Manufacturing on ProcessPlan
                | Activities.
                | Some methods have to be used carefully, they can have no meanly on a specific
                | Activity. This methods allows to support operation from other domain either
                | then Manufacturing.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.manufacturing_activity = com_object

    @property
    def active(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Active() As boolean
                | 
                |     Returns the activation state of the activity.
                | 
                |     Returns:
                |         oActive The activation state of the activity 
                |     Example:
                |         The following example returns in bActive the activation state of the
                |         activity firstActivity:
                | 
                |          Dim firstActivity As ManufacturingActivity
                |          Set firstActivity = ...
                |          Dim bActive As boolean
                |          Set bActive = firstActivity.Active

        :rtype: bool
        """

        return self.manufacturing_activity.Active

    @active.setter
    def active(self, value: bool):
        """
        :param bool value:
        """

        self.manufacturing_activity.Active = value

    @property
    def machinable_feature(self) -> ManufacturingMachinableArea:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property MachinableFeature() As ManufacturingMachinableArea
                | 
                |     Retrieve the machinable area of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retrieves in theMAF the machinable area of the
                |         activity firstActivity.
                | 
                |          Dim firstActivity As ManufacturingActivity
                |          Set firstActivity = ...
                |          Dim theMAF As ManufacturingMachinableArea
                |          Set theMAF = firstActivity.MachinableArea

        :rtype: ManufacturingMachinableArea
        """

        return ManufacturingMachinableArea(self.manufacturing_activity.MachinableFeature)

    @machinable_feature.setter
    def machinable_feature(self, value: ManufacturingMachinableArea):
        """
        :param ManufacturingMachinableArea value:
        """

        self.manufacturing_activity.MachinableFeature = value

    @property
    def machining_time(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property MachiningTime() As double (Read Only)
                | 
                |     Retreive the milling time of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retreives in theMachiningTime the time when the
                |         tool meets the workpiece (in minutes).
                | 
                |          theMachiningTime = firstActivity.MachiningTime

        :rtype: float
        """

        return self.manufacturing_activity.MachiningTime

    @property
    def number_of_feedrate_attributes(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property NumberOfFeedrateAttributes() As short (Read Only)
                | 
                |     This property returns the number of Feedrate attributes of a Manufacturing
                |     Activity.

        :rtype: int
        """

        return self.manufacturing_activity.NumberOfFeedrateAttributes

    @property
    def number_of_geom_attributes(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property NumberOfGeomAttributes() As short (Read Only)
                | 
                |     This property returns the number of Geometry attributes of a Manufacturing
                |     Activity.

        :rtype: int
        """

        return self.manufacturing_activity.NumberOfGeomAttributes

    @property
    def number_of_strategy_attributes(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property NumberOfStrategyAttributes() As short (Read Only)
                | 
                |     This property returns the number of Strategy attributes of a Manufacturing
                |     Activity.

        :rtype: int
        """

        return self.manufacturing_activity.NumberOfStrategyAttributes

    @property
    def precedences(self) -> 'ManufacturingPrecedences':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Precedences() As ManufacturingPrecedences (Read
                | Only)
                | 
                |     This property returns the interface which manages the precedences on the
                |     operation. These collection defines the list of precedences for a manufacturing
                |     Activity.
                | 
                |     Example:
                |         The following example returns in precedences the interface that manage
                |         the precedences for the myActivity Activity :
                | 
                |          ...
                |          Set myActivity  = ... 
                |          Set precedences = myActivity.Precedences

        :rtype: ManufacturingPrecedences
        """
        
        from pycatia.manufacturing_interfaces.manufacturing_precedences import ManufacturingPrecedences
        return ManufacturingPrecedences(self.manufacturing_activity.Precedences)

    @property
    def representation(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Representation() As CATBSTR
                | 
                |     This property returns the path of the representation file for a
                |     manufacturing Activity.

        :rtype: str
        """

        return self.manufacturing_activity.Representation

    @representation.setter
    def representation(self, value: str):
        """
        :param str value:
        """

        self.manufacturing_activity.Representation = value

    @property
    def tool(self) -> ManufacturingTool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Tool() As ManufacturingTool (Read Only)
                | 
                |     Retreive the Tool of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retreives in CurTool the manufacturing tool of
                |         Manufacturing Activity firstActivity
                | 
                |          Set CurTool = firstActivity.GetTool

        :rtype: ManufacturingTool
        """

        return ManufacturingTool(self.manufacturing_activity.Tool)

    @property
    def tool_assembly(self) -> ManufacturingToolAssembly:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property ToolAssembly() As ManufacturingToolAssembly (Read
                | Only)
                | 
                |     Retreive the ToolAssembly of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retrieves in CurrAssembly the manufacturing tool
                |         assembly of the manufacturing activity CurrActivity
                | 
                |          Set CurrAssembly = CurrActivity.ToolAssembly

        :rtype: ManufacturingToolAssembly
        """

        return ManufacturingToolAssembly(self.manufacturing_activity.ToolAssembly)

    @property
    def total_time(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property TotalTime() As double (Read Only)
                | 
                |     Retreive the total Time of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retreives in theTotalTime the total time of the
                |         activity firstActivity (in minutes).
                | 
                |          theTotalTime = firstActivity.TotalTime

        :rtype: float
        """

        return self.manufacturing_activity.TotalTime

    @property
    def video_result(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property VideoResult() As CATBSTR (Read Only)
                | 
                |     Retreive the video result path of a Manufacturing Activity. The path is
                |     empty if no video result.
                | 
                |     Example:
                |         The following example retreives in theVideoResult the video result of
                |         the activity firstActivity (in minutes).
                | 
                |          theVideoResult = firstActivity.VideoResult

        :rtype: str
        """

        return self.manufacturing_activity.VideoResult

    def get_attribute(self, i_attribut: str) -> Parameter:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetAttribute(CATBSTR iAttribut) As Parameter
                | 
                |     Retreive the Attribute of a Manufacturing Activity.
                | 
                |     Example:
                |         The following example retreives in RapidFeed the attribute MfgRapidFeed
                |         of Manufacturing Activity firstActivity
                | 
                |          Set RapidFeed = firstActivity.GetAttribute(MfgRapidFeed)

        :param str i_attribut:
        :rtype: Parameter
        """
        return Parameter(self.manufacturing_activity.GetAttribute(i_attribut))

    def get_attribute_nls_name(self, i_attribut_name: str) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetAttributeNLSName(CATBSTR iAttributName) As CATBSTR
                | 
                |     Retrieve the NLS name from the attribute name of a Manufacturing
                |     Activity.

        :param str i_attribut_name:
        :rtype: str
        """
        return self.manufacturing_activity.GetAttributeNLSName(i_attribut_name)

    def get_list_of_feedrate_attributes(self, o_list_of_attributes: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetListOfFeedrateAttributes(CATSafeArrayVariant
                | oListOfAttributes)
                | 
                |     Retrieve the list of Feedrate attributes of a Manufacturing
                |     Activity.
                |     Each attribute is returned as the name of a CKE object.
                | 
                |     Example:
                |         The following example retrieves in oListOfAttributes the list of
                |         feedrate attributes of the Manufacturing Activity
                |         myActivity
                | 
                |          call myActivity.GetListOfFeedrateAttributes(oListOfAttributes)

        :param tuple o_list_of_attributes:
        :rtype: None
        """
        return self.manufacturing_activity.GetListOfFeedrateAttributes(o_list_of_attributes)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_list_of_feedrate_attributes'
        # # vba_code = """
        # # Public Function get_list_of_feedrate_attributes(manufacturing_activity)
        # #     Dim oListOfAttributes (2)
        # #     manufacturing_activity.GetListOfFeedrateAttributes oListOfAttributes
        # #     get_list_of_feedrate_attributes = oListOfAttributes
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_list_of_geom_attributes(self, o_list_of_attributes: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetListOfGeomAttributes(CATSafeArrayVariant
                | oListOfAttributes)
                | 
                |     Retrieve the list of Geometry attributes of a Manufacturing
                |     Activity.
                |     Each attribute is returned as the name of a CKE object.
                | 
                |     Example:
                |         The following example retrieves in oListOfAttributes the list of
                |         Geometry attributes of the Manufacturing Activity
                |         myActivity
                | 
                |          call
                |          myActivity.GetListOfGeomAttributes(oListOfAttributes)

        :param tuple o_list_of_attributes:
        :rtype: None
        """
        return self.manufacturing_activity.GetListOfGeomAttributes(o_list_of_attributes)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_list_of_geom_attributes'
        # # vba_code = """
        # # Public Function get_list_of_geom_attributes(manufacturing_activity)
        # #     Dim oListOfAttributes (2)
        # #     manufacturing_activity.GetListOfGeomAttributes oListOfAttributes
        # #     get_list_of_geom_attributes = oListOfAttributes
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_list_of_strategy_attributes(self, o_list_of_attributes: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetListOfStrategyAttributes(CATSafeArrayVariant
                | oListOfAttributes)
                | 
                |     Retrieve the list of Strategy attributes of a Manufacturing
                |     Activity.
                |     Each attribute is returned as the name of a CKE object.
                | 
                |     Example:
                |         The following example retrieves in oListOfAttributes the list of
                |         Strategy attributes of the Manufacturing Activity
                |         myActivity
                | 
                |          call myActivity.GetListOfStrategyAttributes(oListOfAttributes)

        :param tuple o_list_of_attributes:
        :rtype: None
        """
        return self.manufacturing_activity.GetListOfStrategyAttributes(o_list_of_attributes)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_list_of_strategy_attributes'
        # # vba_code = """
        # # Public Function get_list_of_strategy_attributes(manufacturing_activity)
        # #     Dim oListOfAttributes (2)
        # #     manufacturing_activity.GetListOfStrategyAttributes oListOfAttributes
        # #     get_list_of_strategy_attributes = oListOfAttributes
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_machining_direction(self, o_x_axis: float, o_y_axis: float, o_z_axis: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetMachiningDirection(double oXAxis,
                | double oYAxis,
                | double oZAxis)
                | 
                |     Retreives the Machining Direction coordinates of a Manufacturing
                |     Operation.

        :param float o_x_axis:
        :param float o_y_axis:
        :param float o_z_axis:
        :rtype: None
        """
        return self.manufacturing_activity.GetMachiningDirection(o_x_axis, o_y_axis, o_z_axis)

    def get_tool_axis(self, o_x_axis: float, o_y_axis: float, o_z_axis: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetToolAxis(double oXAxis,
                | double oYAxis,
                | double oZAxis)
                | 
                |     Retreives the ToolAxis coordinates of a Manufacturing
                |     Activity.

        :param float o_x_axis:
        :param float o_y_axis:
        :param float o_z_axis:
        :rtype: None
        """
        return self.manufacturing_activity.GetToolAxis(o_x_axis, o_y_axis, o_z_axis)

    def set_machining_direction(self, i_x_axis: float, i_y_axis: float, i_z_axis: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetMachiningDirection(double iXAxis,
                | double iYAxis,
                | double iZAxis)
                | 
                |     Defines the Machining Direction coordinates of a Manufacturing
                |     Operation.

        :param float i_x_axis:
        :param float i_y_axis:
        :param float i_z_axis:
        :rtype: None
        """
        return self.manufacturing_activity.SetMachiningDirection(i_x_axis, i_y_axis, i_z_axis)

    def set_tool_axis(self, i_x_axis: float, i_y_axis: float, i_z_axis: float) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetToolAxis(double iXAxis,
                | double iYAxis,
                | double iZAxis)
                | 
                |     Defines the ToolAxis coordinates of a Manufacturing
                |     Activity.

        :param float i_x_axis:
        :param float i_y_axis:
        :param float i_z_axis:
        :rtype: None
        """
        return self.manufacturing_activity.SetToolAxis(i_x_axis, i_y_axis, i_z_axis)

    def __repr__(self):
        return f'ManufacturingActivity(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from typing import TYPE_CHECKING

from pathlib import Path

from pycatia.exception_handling import CATIAApplicationException
from pycatia.in_interfaces.cameras import Cameras
from pycatia.in_interfaces.reference import Reference
from pycatia.in_interfaces.window import Window
from pycatia.in_interfaces.workbench import Workbench
from pycatia.space_analyses_interfaces.spa_workbench import SPAWorkbench
from pycatia.system_interfaces.any_object import AnyObject

if TYPE_CHECKING:
    from pycatia.in_interfaces.selection import Selection


class Document(AnyObject):
    """
        .. note::
            :class: toggle pytoggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Document
                |
                | Represents the document.
                | The document is the object handled by the operating system as a whole that
                | stores your data in files and databases. It is assigned a type determined by
                | its contents. It may contain other documents with a different type. For
                | example, a PartDocument contains a part and can be contained in a
                | ProductDocument. A workshop is associated with a document to gather all the
                | commands that can be used to create, modify, and edit the objects making up the
                | the document. These commands are arranged in menus and
                | toolbars.
                |
                | See also:
                |     PartDocument, ProductDocument, DrawingDocument

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.document = com_object

    @property
    def cameras(self) -> Cameras:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Cameras() As Cameras (Read Only)
                |
                |     Returns the document's collection of cameras.
                |
                |     Example:
                |         This example retrieves in CameraCollection the collection of cameras
                |         attached to the Doc document.
                |
                |          Dim CameraCollection As Cameras
                |          Set CameraCollection = Doc.Cameras

        :rtype: Cameras
        """

        return Cameras(self.document.Cameras)

    @property
    def current_filter(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property CurrentFilter() As CATBSTR
                |
                |     Returns or sets the current visualization filter. CurrentFilter uses the
                |     filter name and not its definition. The "All visible" filter means that all
                |     layers are visible. For all filters, remind that the current layer is always
                |     visible.
                |
                |     Example:
                |         This example makes the filter named "Filter001" as the current
                |         visualization filter for the Doc document.
                |
                |          Doc.CurrentFilter = "Filter001"

        :rtype: str
        """

        return self.document.CurrentFilter

    @current_filter.setter
    def current_filter(self, value: str):
        """
        :param str value:
        """

        self.document.CurrentFilter = value

    @property
    def current_layer(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property CurrentLayer() As CATBSTR
                |
                |     Returns or sets the current layer. CurrentLayer uses the layer name and not
                |     its number. The "None" layer means that there is no current
                |     layer.
                |
                |     Example:
                |         This example makes the layer named "Layer 3" as the current layer for
                |         the Doc document.
                |
                |          Doc.CurrentLayer = "Layer 3"

        :rtype: str
        """

        return self.document.CurrentLayer

    @current_layer.setter
    def current_layer(self, value: str):
        """
        :param str value:
        """

        self.document.CurrentLayer = value

    @property
    def is_part(self) -> bool:
        """
        Determine whether the active document is a CATPart.

        :return: bool
        """
        try:
            if self.part:
                return True
        except AttributeError:
            return False

    @property
    def is_product(self) -> bool:
        """
        Determine whether the active document is a CATProduct.

        :return: bool
        """
        if self.product.is_catproduct():
            return True
        return False

    @property
    def is_saved(self) -> bool:
        """
        Returns true if document is saved.

        .. note::
            :class: toggle

            CAA V5 Visual Basic help

            Property Saved( ) As boolean (Read Only)

            Returns whether the document has been modified, and thus needs to be saved.
            This happens when the document has changed since either its creation or its last save.
            True if the document has not been changed: the document doesn't need to be saved.
            False if the document has been changed: the document needs to be saved.

            | Example:
            | This example retrieves in HasChanged whether the Doc document needs to be saved.
            | HasChanged = NOT Doc.Saved

        :return: bool
        """

        return self.document.Saved

    @property
    def full_name(self) -> str:
        """

        .. note::
            :class: toggle

            CAA V5 Visual Basic help

            Property FullName( ) As CATBSTR (Read Only)

            | Returns the document's full file name, including its path.
            | Example:
            | This example retrieves in DocFullName the Doc document's full file name.
            |     DocFullName = Doc.FullName
            |
            | The returned value is like this:
            |     e://users//psr//Parts//MyNicePart.CATPart


        :return: str - full path document name
        """

        return str(self.path())

    @property
    def read_only(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property ReadOnly() As boolean (Read Only)
                |
                |     Returns whether the file containing the document can be read only, on can
                |     be read and written.
                |     True if the file is read-only.
                |
                |     Example:
                |         This example retrieves in IsReadOnly the ability to read, and possibly
                |         to write in, the file containing the Doc document.
                |
                |          IsReadOnly = Doc.ReadOnly

        :rtype: bool
        """

        return self.document.ReadOnly

    @property
    def see_hidden_elements(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property SeeHiddenElements() As boolean
                |
                |     Returns or sets the document's hidden elements visibility.
                |     True if the document's hidden elements are visible to the
                |     user.
                |
                |     Example:
                |         This example makes the Doc document's hidden elements
                |         visible.
                |
                |          Doc.SeeHiddenElements = True

        :return: bool
        """

        return self.document.SeeHiddenElements

    @see_hidden_elements.setter
    def see_hidden_elements(self, value: bool):
        """
        :param bool value:
        """

        self.document.SeeHiddenElements = value

    @property
    def selection(self) -> 'Selection':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Selection() As Selection (Read Only)
                |
                |     Returns the current selection. The current selection is the object or the
                |     set of objects the end user has selected, usually with the mouse, in the active
                |     document displayed in the active window.
                |
                |     Example:
                |         This example returns in CurSel the current selection in the Doc
                |         document
                |
                |          Dim CurSel As Selection
                |          Set CurSel = Doc.Selection

        :rtype: Selection
        """
        from pycatia.in_interfaces.selection import Selection
        return Selection(self.document.Selection)

    def activate(self) -> None:
        """
        Activates the document

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Activate()
                |
                |     Activates the document. Activating a document means that this document is
                |     the one on which the end user is now working on. This document possibly
                |     reconfigures the menu bar and toolbars with its own commands if its type is
                |     different from the type of the previous active document. The first window in
                |     the window collection which contains this document becomes the active
                |     one.
                |
                |     Example:
                |         This example activates the Doc document.
                |
                |          Doc.Activate()

        :rtype: None
        """

        return self.document.Activate()

    def close(self) -> None:
        """
        Closes the current document.

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Close()
                |
                |     Closes the document. This closes all the windows displaying the document.
                |     If the document needs to be saved, the end user is prompted whether to save the
                |     document, or to close it anyway.
                |
                |     Example:
                |         This example closes the Doc document
                |
                |          Doc.Close()

        """
        self.document.Close()

    def create_filter(self, i_filter_name: str, i_filter_definition: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub CreateFilter(CATBSTR iFilterName,
                | CATBSTR iFilterDefinition)
                |
                |     Creates a new visualization filter from a name and a definition. Fails if
                |     there is already a filter named iFilterName.
                |
                |     Parameters:
                |
                |         iFilterName
                |             The filter name.
                |         iFilterDefinition
                |             The filter definition
                |         Example:
                |             This example creates the filter named "Filter001" and with "layer=
                |             2 & layer= 1" definition for the Doc document.
                |
                |              Doc.CreateFilter ("Filter001", "layer= 2 & layer=
                |              1")

        :param str i_filter_name:
        :param str i_filter_definition:
        :rtype: None
        """
        return self.document.CreateFilter(i_filter_name, i_filter_definition)

    def create_reference_from_name(self, i_label: str) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateReferenceFromName(CATBSTR iLabel) As Reference
                |
                |     Creates a reference from a GenericNaming label. Each kind of document
                |     provides a specific implementation.
                |
                |     Parameters:
                |
                |         iLabel
                |             The GenericNaming identification for an object.
                |
                |     Returns:
                |         The reference to the object.

        :param str i_label:
        :rtype: Reference
        """
        return Reference(self.document.CreateReferenceFromName(i_label))

    def get_workbench(self, workbench_name: str) -> Workbench:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetWorkbench(CATBSTR workbenchName) As Workbench
                |
                |     Returns one of the workbenches of the document.
                |
                |     Parameters:
                |
                |         workbenchName
                |             The name of the workbench
                |
                |             Example:
                |                 This example retrieves the Structural workbench on the Doc
                |                 document
                |
                |                  Doc.GetWorkbench("Structural")

        :param str workbench_name:
        :rtype: Workbench
        """
        return Workbench(self.document.GetWorkbench(workbench_name))

    def export_data(self, file_name: Path, file_type: str, overwrite=False) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic help

            Sub ExportData( CATBSTR  fileName, CATBSTR  format)

            | Exports the data contained in the document to another format.
            | Parameters:
            |   fileName
            |       The name of the exported file
            |   format
            |       The name of the format
            | Example:
            |   This example writes the Doc document in the IGES format under the IGESDoc name.
            |       Doc.ExportData("IGESDoc", "igs")


        :param Path file_name: file_name including full path.
        :param str file_type: file_type is the extension of required file_type.
                              The file_type must be supported by CATIA and the CATIA license.
        :param bool overwrite: Files will not be overwritten unless is True.
        :return:
        """
        current_dfa_setting = self.application.display_file_alerts
        if not isinstance(file_name, Path):
            file_name = Path(file_name)

        if file_name.suffix.lower() != '.' + file_type.lower():
            raise CATIAApplicationException(
                f'Filename "{file_name}" must have the same suffix as filetype "{file_type}".')

        # add filetype to filename if it hasn't been added correctly.
        if not str(file_name).endswith(file_type):
            file_name = Path(f"{file_name}.{file_type}")

        if not file_name.parent.is_dir():
            raise NotADirectoryError(f'Directory: {file_name.parent} is not a directory.')

        if overwrite is False:
            if file_name.is_file():
                raise FileExistsError(f'File: {file_name} already exists. '
                                      f'Set overwrite=True if you want to overwrite.')
        else:
            self.application.display_file_alerts = False

        # pycatia prefers full path names :-)
        if not file_name.is_absolute():
            self.logger.warning('To prevent unexpected behaviour, be explicit and use absolute filenames.')

        self.document.ExportData(file_name, file_type)

        self.application.display_file_alerts = current_dfa_setting

    def indicate_2d(self, i_message: str) -> str:
        """

        Returns a tuple in the format (status: str, (x_2d: float, y_2d: float))
        Status can be 'Normal', 'Cancel',

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Indicate2D(CATBSTR iMessage,
                | CATSafeArrayVariant ioDocumentWindowLocation) As CATBSTR
                |
                |     Runs an 2D interactive indication command.
                |     Role: Indicate2D asks to the user to select a location in the document
                |     window. It can be used:
                |     When this document is a DrawingDocument
                |
                |     When this document is a PartDocument, and a sketch is being edited (
                |     Sketch.OpenEdition has been called and Sketch.CloseEdition has not been called
                |     yet)
                |
                |     See also: Selection.IndicateOrSelectElement2D which can, in particular,
                |     enable indication and not selection (positionning the iFilterType parameter to
                |     an empty string), whichs enables to subscribe to mouse move events,
                |     positionning the iTriggeringOnPreSelection to true.
                |     Note:If the scripting language is Visual Basic for Applications or Visual
                |     Basic 6 Development Studio, then, you have to know that during the execution of
                |     an interactive selection method such as this one, no form (dialog box) must be
                |     displayed, otherwise it would lead to unpredictible results. In a form method,
                |     before calling an interactive selection method such as Document.Indicate2D, you
                |     must hide all forms, and, after the call to the method, you must show the
                |     forms.
                |
                |     Parameters:
                |
                |         iMessage
                |             A string which instructs the user that he must select a location in
                |             the document window. This string is displayed in the message area located at
                |             the left of the power input area.
                |         oDocumentWindowLocation
                |             An array made of 2 doubles: X, Y - coordinates array of the
                |             location the user specified in the document window.
                |
                |         oOutputState
                |             The state of the indication command once Indicate2D returns. It can
                |             be either "Normal" (the indication has succeeded), "Cancel" (the user wants to
                |             cancel the VB command, which must exit immediately, see the oOutputState
                |             parameter of the Selection.SelectElement2 method), "Undo" or "Redo". About the use of
                |             "Undo" and "Redo", see the example of the Selection.SelectElement2 method.
                |
                |     Example:
                |
                |           The following example suppose a drawing document is currently edited.
                |           It asks the end user to select a
                |          location in the current drawing window, and creates a text (see
                |
                |
                |     DrawingText ) at the specified location:
                |
                |      Set Document = CATIA.ActiveDocument
                |      Set Selection = Document.Selection
                |      Set DrawingSheets = Document.Sheets
                |      Set DrawingSheet = DrawingSheets.ActiveSheet
                |      Set DrawingViews = DrawingSheet.Views
                |      Set DrawingView = DrawingViews.ActiveView
                |      Set DrawingTexts = DrawingView.Texts
                |      'We propose to the user that he specify a location in the drawing window
                |      Dim DrawingWindowLocation(1)
                |      Status=Document.Indicate2D("select a location into the drawing
                |      window",DrawingWindowLocation)
                |      if (Status = "Cancel") then Exit Sub
                |      Set DrawingText=DrawingTexts.Add("Hello
                |      world",DrawingWindowLocation(0),DrawingWindowLocation(1))

        :param str i_message:
        :return: tuple
        """

        vba_function_name = 'indicate_2d'
        vba_code = f'''
        Public Function {vba_function_name}(document, i_message)
        Dim DrawingWindowLocation (1)
        Dim o_output(1)
        o_output(0) = document.Indicate2D(i_message, DrawingWindowLocation)
        o_output(1) = DrawingWindowLocation
        {vba_function_name} = o_output
        End Function
        '''

        system_service = self.application.system_service
        return system_service.evaluate(
            vba_code,
            0,
            vba_function_name,
            [
                self.document,
                i_message
            ]
        )

    def indicate_3d(
            self,
            i_planar_geometric_object: AnyObject,
            i_message: str,
    ) -> str:
        """

        Returns a tuple in the format (status: str, (x_2d: float, y_2d: float), (x_3d: float, y_3d: float, z_3d: float))
        Status can be 'Normal', 'Cancel',

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func Indicate3D(AnyObject iPlanarGeometricObject,
                | CATBSTR iMessage,
                | CATSafeArrayVariant ioWindowLocation2D,
                | CATSafeArrayVariant ioWindowLocation3D) As CATBSTR
                |
                |     Runs an 3D interactive indication command.
                |     Role: Indicate3D asks to the user to select a location in the document
                |     window. It cannot be used:
                |     When this document is a DrawingDocument
                |
                |     When this document is a PartDocument, and a sketch is being edited (
                |     Sketch.OpenEdition has been called and Sketch.CloseEdition has not been called
                |     yet)
                |
                |     In these cases, Indicate2D must be used.
                |     See also: Selection.IndicateOrSelectElement3D which can, in particular,
                |     enable indication and not selection (positioning the iFilterType parameter to
                |     an empty string), which enables to subscribe to mouse move events,
                |     positioning the iTriggeringOnPreSelection to true.
                |     Note:If the scripting language is Visual Basic for Applications or Visual
                |     Basic 6 Development Studio, then, you have to know that during the execution of
                |     an interactive selection method such as this one, no form (dialog box) must be
                |     displayed, otherwise it would lead to unpredictable results. In a form method,
                |     before calling an interactive selection method such as Document.Indicate2D, you
                |     must hide all forms, and, after the call to the method, you must show the
                |     forms.
                |
                |     Parameters:
                |
                |         iPlanarGeometricObject
                |             A planar geometric object.
                |             The following objects are supported:
                |
                |         HybridShapeCircle, HybridShapeCircleExplicit, HybridShapeConic, Sketch,
                |         Circle2D, Ellipse2D, Hyperbola2D, Parabola2D and Spline2D.
                |
                |     iMessage
                |         A string which instructs the user that he must select a location in the
                |         document window. This string is displayed in the message area located at the
                |         left of the power input area.
                |     oWindowLocation2D
                |         An array made of 2 doubles: X, Y - coordinates array of the location
                |         the user specified in the document window, in the input planar object
                |         coordinates system
                |     oWindowLocation3D
                |         An array made of 3 doubles: X, Y, Z - coordinates array of the location
                |         the user specified in the document window
                |     oOutputState
                |         The state of the indication command once Indicate3D returns. It can be
                |         either "Normal" (the indication has succeeded), "Cancel" (the user wants to
                |         cancel the VB command, which must exit immediately, see the oOutputState
                |         parameter of the Selection.SelectElement2 method), "Undo" or "Redo". About the
                |         use of "Undo" and "Redo", see the example of the Selection.SelectElement2
                |         method.
                |     Example:
                |
                |           The following example asks the end user to select a location in the
                |
                |          document window, on the Plane.1 plane, and creates a
                |
                |     HybridShapePointOnPlane at the specified location:
                |
                |      Set Document = CATIA.ActiveDocument
                |      Set Part  = Document.Part
                |      Set Selection = Document.Selection
                |      Set HybridShapeFactory = Part.HybridShapeFactory
                |      Set HybridShapePlane = Part.Bodies.Item("PartBody").HybridShapes.Item("Plane.1")
                |      Set PlaneReference = Part.CreateReferenceFromObject(HybridShapePlane)
                |      'We propose to the user that he select a location in the
                |      window
                |      ReDim WindowLocation2D(1),WindowLocation3D(2)
                |      Status=Document.Indicate3D(HybridShapePlane,"select a location in the
                |      document window", _
                |      WindowLocation2D,WindowLocation3D)
                |      if (Status = "Cancel") then Exit Sub
                |      Set HybridShapePointOnPlane = HybridShapeFactory.AddNewPointOnPlane( _
                |      PlaneReference,WindowLocation2D(0),WindowLocation2D(1))
                |      Part.Bodies.Item("PartBody").InsertHybridShape
                |      HybridShapePointOnPlane
                |      Part.InWorkObject = HybridShapePointOnPlane
                |      Part.Update

        :param AnyObject i_planar_geometric_object:
        :param str i_message:
        :return: str
        """
        vba_function_name = 'indicate_3d'
        vba_code = f'''
        Public Function {vba_function_name}(document, i_planar_geometric_object, i_message)
        Dim WindowLocation2D (1)
        Dim WindowLocation3D (2)
        Dim o_output(2)
        o_output(0) = document.Indicate3D(i_planar_geometric_object, i_message, WindowLocation2D, WindowLocation3D)
        o_output(1) = WindowLocation2D
        o_output(2) = WindowLocation3D
        {vba_function_name} = o_output
        End Function
        '''

        system_service = self.application.system_service
        return system_service.evaluate(
            vba_code,
            0,
            vba_function_name,
            [
                self.document,
                i_planar_geometric_object.com_object,
                i_message
            ]
        )

    def new_window(self) -> Window:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func NewWindow() As Window
                |
                |     Duplicates the window which contains the active document. This implies
                |     creating a window, displaying the active document in this window with the same
                |     view point, making this window the active one, and adding the window to the
                |     collection of windows.
                |
                |     Example:
                |         This example creates the MyWindow new window for the Doc
                |         document.
                |
                |          Dim MyWindow As Window
                |          Set MyWindow = Doc.NewWindow()

        :return: Window
        """
        return Window(self.document.NewWindow())

    def path(self) -> Path:
        """

        Returns the pathlib.Path() object of the document fullname.

        example e://users//psr//Parts//MyNicePart.CATPart
        >>> Document.path().name
        MyNicePart.CATPart
        >>> Document.path().parent
        e://users//psr//Parts//
        >>> Document.path().suffix
        .CATPart

        :return: Path()
        """

        return Path(self.document.FullName)

    def remove_filter(self, i_filter_name) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub RemoveFilter(CATBSTR iFilterName)
                |
                |     Removes an existing visualization filter. Fails if the filter to be removed
                |     is the current filter.
                |
                |     Parameters:
                |
                |         iFilterName
                |             The filter name.
                |         Example:
                |             This example removes the filter named "Filter001" for the Doc
                |             document.
                |
                |              Doc.RemoveFilter ("Filter001")

        :param str i_filter_name:
        :rtype: None
        """
        return self.document.RemoveFilter(i_filter_name)

    def save(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Save()
                |
                |     Saves the document.
                |
                |     Example:
                |         This example saves the Doc document.
                |
                |          Doc.Save()

        :rtype: None
        """
        self.logger.info(f'Saving the current document.')
        self.document.Save()

    def save_as(self, file_name: Path, overwrite: bool = False) -> None:
        """
        Save the document to a new name.

        If overwrite is True CAA.DisplayFileAlerts is set to False.

        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub SaveAs(CATBSTR fileName)
                |
                |     Saves the document with another name.
                |
                |     Parameters:
                |
                |         fileName
                |             The name to assign to the document
                |
                |     Example:
                |         This example saves the Doc document with the NewName
                |         name.
                |
                |          Doc.SaveAs("NewName")

        :param file_name: full pathname to new file_name
        :param bool overwrite: Files will not be overwritten unless is True.
        :return: None.
        :rtype: None
        """
        current_dfa_setting = self.application.display_file_alerts
        path_file_name = Path(file_name)

        # pycatia prefers full path names :-)
        if not path_file_name.is_absolute():
            self.logger.warning('To prevent unexpected behaviour, be explicit and use absolute filenames.')

        if overwrite is False:
            if path_file_name.is_file():
                raise FileExistsError(f'File: {path_file_name} already exists. '
                                      f'Set overwrite=True if you want to overwrite.')
        else:
            self.application.display_file_alerts = False
            # if path_file_name.is_file():
            #     self.logger.warning('File already exists. Click YES in CATIA V5.')

        self.document.SaveAs(path_file_name)

        self.application.display_file_alerts = current_dfa_setting

    def search_for_items(self, selection_objects):
        """
        # todo: This search is currently restricted to GSD objects only.
        Selection objects is a list of items to search for.
        Example: selection_objects = ['Point', 'Line']
        Example query string to search for all lines and points
        "('Generative Shape Design'.Point + 'Generative Shape Design'.Line),in"
        :param list selection_objects:
        :return Selected Automation Object:
        """

        self.logger.warning('This method may be deprecated in future versions.')

        gsd_items = [
            'Point',
            'Line'
        ]

        query_string = str()
        # build query string

        for counter, item in enumerate(selection_objects):
            boolean = str()
            if counter > 0 and not counter == len(selection_objects):
                boolean = ' + '
            if item in gsd_items:
                query_string = f"{query_string}{boolean}'Generative Shape Design'.{item}"

        query_string = f"({query_string}),in"

        selection = self.document.Selection
        selection.Search(query_string)

        selected = list()
        for i in range(0, selection.Count):
            selected.append(AnyObject(selection.Item(i + 1).Value))

        return selected

    def spa_workbench(self) -> SPAWorkbench:
        """
        :return:
        :rtype: SPAWorkbench
        """

        return SPAWorkbench(self.com_object)

    def __repr__(self):
        return f'Document(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_str_functional_interfaces.sfm_profile import SFMProfile
from pycatia.cat_str_functional_interfaces.sfm_welds import SFMWelds
from pycatia.in_interfaces.reference import Reference


class SFMStiffenerOnFreeEdge(SFMProfile):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     CATStrFunctionalInterfaces.SfmObject
                |                         CATStrFunctionalInterfaces.SfmProfile
                |                             SfmStiffenerOnFreeEdge
                | 
                | Interface to manage different attributes of Stiffener on free
                | edge.
                | Role: Provides access to Structural Functional Modeler's Stiffener on free
                | edge.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sfm_stiffener_on_free_edge = com_object

    @property
    def free_edge_offset(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property FreeEdgeOffset() As double
                | 
                |     Returns or sets the SFE's Free Edge offset.
                | 
                |     Parameters:
                | 
                |         oOffset
                |             [out] The retrieved Free Edge offset. 
                |         iOffset
                |             [in] The input Free Edge offset. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example retrieves in Offset the Free Edge offset of the
                |             SfmStiffenerOnFreeEdge feature.
                | 
                |              Dim Offset As Double
                |              Set Offset = SfmStiffenerOnFreeEdge.FreeEdgeOffset

        :rtype: float
        """

        return self.sfm_stiffener_on_free_edge.FreeEdgeOffset

    @free_edge_offset.setter
    def free_edge_offset(self, value: float):
        """
        :param float value:
        """

        self.sfm_stiffener_on_free_edge.FreeEdgeOffset = value

    @property
    def section_angle(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property SectionAngle() As double
                | 
                |     Returns or sets the SFE's section angle.
                | 
                |     Parameters:
                | 
                |         oAngle
                |             [out] The retrieved angle of the section of SFE. 
                |         iAngle
                |             [in] The retrieved angle of the section of SFE. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example retrieves in Angle the angle of the section of
                |             SfmStiffenerOnFreeEdge feature.
                | 
                |              Dim Angle As Double
                |              Set Angle = SfmStiffenerOnFreeEdge.SectionAngle

        :rtype: float
        """

        return self.sfm_stiffener_on_free_edge.SectionAngle

    @section_angle.setter
    def section_angle(self, value: float):
        """
        :param float value:
        """

        self.sfm_stiffener_on_free_edge.SectionAngle = value

    @property
    def side_orientation(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property SideOrientation() As long
                | 
                |     Returns or sets the side orientation.
                | 
                |     Parameters:
                | 
                |         oOrientation
                |             [out] The retrieved Side Orientation of SFE. 
                |         iOrientation
                |             [in] The input Side Orientation of SFE. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example retrieves side orientation of the
                |             Stiffener.
                | 
                |              Dim Orient As Long
                |              Orient = SfmStiffenerOnFreeEdge.SideOrientation

        :rtype: int
        """

        return self.sfm_stiffener_on_free_edge.SideOrientation

    @side_orientation.setter
    def side_orientation(self, value: int):
        """
        :param int value:
        """

        self.sfm_stiffener_on_free_edge.SideOrientation = value

    def get_free_edge(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetFreeEdge() As Reference
                | 
                |     Returns the Free Edge.
                | 
                |     Parameters:
                | 
                |         oFreeEdge
                |             [out] The retrieved Free Edge. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example retrieves in FreeEdge the Free Edge of the
                |             SfmStiffenerOnFreeEdge feature.
                | 
                |              Dim FreeEdge As Reference
                |              Set FreeEdge = SfmStiffenerOnFreeEdge.GetFreeEdge

        :rtype: Reference
        """
        return Reference(self.sfm_stiffener_on_free_edge.GetFreeEdge())

    def get_molded_surface(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetMoldedSurface() As Reference
                | 
                |     Returns the molded surface.
                | 
                |     Parameters:
                | 
                |         oMoldedSurface
                |             [out] The retrieved Molded Surface. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example retrieves MoldedSurface the molded surface of the
                |             SfmStiffenerOnFreeEdge feature.
                | 
                |              Dim MoldedSurface As Reference
                |              Set MoldedSurface = SfmStiffenerOnFreeEdge.GetMoldedSurface

        :rtype: Reference
        """
        return Reference(self.sfm_stiffener_on_free_edge.GetMoldedSurface())

    def get_welds(self, i_operating_ele: Reference) -> SFMWelds:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetWelds(Reference iOperatingEle) As SfmWelds
                | 
                |     Gets Welds feature on operated SFE.
                | 
                |     Parameters:
                | 
                |         iOperatingEle
                |             [in] Operating element of the weld features. 
                |         oWelds
                |             [out] The retrieved Weld features. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example gets welds features of Split SFE.
                | 
                |              Dim Welds As SfmWelds
                |              Set Welds = SplitSFE.GetWelds(Nothing)

        :param Reference i_operating_ele:
        :rtype: SFMWelds
        """
        return SFMWelds(self.sfm_stiffener_on_free_edge.GetWelds(i_operating_ele.com_object))

    def set_free_edge(self, i_free_edge: Reference) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetFreeEdge(Reference iFreeEdge)
                | 
                |     Sets the Free Edge.
                | 
                |     Parameters:
                | 
                |         iFreeEdge
                |             [in] The input Free Edge. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example:
                |             This example sets FreeEdge the Free Edge of the
                |             SfmStiffenerOnFreeEdge feature.
                | 
                |              Dim FreeEdge As Reference
                |              Set FreeEdge = SfmStiffenerOnFreeEdge.SetFreeEdge

        :param Reference i_free_edge:
        :rtype: None
        """
        return self.sfm_stiffener_on_free_edge.SetFreeEdge(i_free_edge.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_free_edge'
        # # vba_code = """
        # # Public Function set_free_edge(sfm_stiffener_on_free_edge)
        # #     Dim iFreeEdge (2)
        # #     sfm_stiffener_on_free_edge.SetFreeEdge iFreeEdge
        # #     set_free_edge = iFreeEdge
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'SFMStiffenerOnFreeEdge(name="{self.name}")'



#! /usr/bin/python3.9

from win32com.client import Dispatch
import pythoncom

from pycatia.in_interfaces.application import Application


def catia_application(co_initialise=False) -> Application:
    if co_initialise:
        return Application(Dispatch('CATIA.Application', pythoncom.CoInitialize()))
    return Application(Dispatch('CATIA.Application'))



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.funct_system_interfaces.functional_variant import FunctionalVariant
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class FunctionalVariants(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     FunctionalVariants
                | 
                | The interface to access a collection of FunctionalVariants.
    
    """

    def __init__(self, com_object, child_object=FunctionalVariant):
        super().__init__(com_object, child_object=FunctionalVariant)
        self.functional_variants = com_object
        self.child_object = child_object

    def create(self, i_name: str) -> FunctionalVariant:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Create(CATBSTR iName) As FunctionalVariant
                | 
                |     Create a FunctionalVariant.

        :param str i_name:
        :rtype: FunctionalVariant
        """
        return FunctionalVariant(self.functional_variants.Create(i_name))

    def delete(self, i_variant: FunctionalVariant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Delete(FunctionalVariant iVariant)
                | 
                |     Delete a FunctionalVariant.

        :param FunctionalVariant i_variant:
        :rtype: None
        """
        return self.functional_variants.Delete(i_variant.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'delete'
        # # vba_code = """
        # # Public Function delete(functional_variants)
        # #     Dim iVariant (2)
        # #     functional_variants.Delete iVariant
        # #     delete = iVariant
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def elem(self, i_index: cat_variant) -> FunctionalVariant:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Elem(CATVariant iIndex) As FunctionalVariant
                | 
                |     Returns a Variant using its index or its name from the Variants
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Variant to retrieve from the
                |             collection of Variants. As a numerics, this index is the rank of the Variant in
                |             the collection. The index of the first Variant in the collection is 1, and the
                |             index of the last Variant is Count. As a string, it is the name you assigned to
                |             the Variant using the 
                | 
                |         AnyObject.Name property. 
                |     Returns:
                |         The retrieved Variant 
                |     Example:
                |         This example retrieves in Act1 the fifth Variant in the collection and
                |         in Act2 the Variant named Moves.
                | 
                |          Dim Act1 As FunctionalVariant
                |          Set Act1 = Desc.Variant(5)
                |          Dim Act2 As FunctionalVariant
                |          Set Act2 = Desc.Variant("Adding new substance")

        :param cat_variant i_index:
        :rtype: FunctionalVariant
        """
        return FunctionalVariant(self.functional_variants.Elem(i_index))

    def __repr__(self):
        return f'FunctionalVariants(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject


class ExpertReportObject(AnyObject):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     ExpertReportObject
                | 
                | Represents the ExpertReportObject object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.expert_report_object = com_object

    @property
    def validity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Validity() As boolean (Read Only)
                | 
                |     Returns the validity of a check for a given tuple. The result depends on
                |     the result of the check for a given tuple :
                | 
                |     "True" or "False" for the tuple.

        :rtype: bool
        """

        return self.expert_report_object.Validity

    def get_tuple(self, o_safe_array: tuple) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub getTuple(CATSafeArrayVariant oSafeArray)
                | 
                |     Returns a tuple (a collection of objects) concerned by this report
                |     object.
                | 
                |     Example:
                | 
                |          Dim relations1 As Relations
                |          Dim RuleBase1 As ExpertRuleBaseRuntime
                |          Dim ListComponents As ExpertRuleBaseComponentRuntimes
                |          Dim AComponent As ExpertRuleBaseComponentRuntime
                |          Dim NupletsList As ExpertReportObjects
                |          Dim ANuplet As ExpertReportObject
                |          Dim anArray () as Object
                |          Dim anElementArray as Object
                | 
                |          Set relations1 = part1.Relations
                |          Set RuleBase1 = relations1.Item("RuleBase")
                |          Set ListComponents = RuleBase1.RuleSet.ExpertRuleBaseComponentRuntimes
                |          ' Let's get a check ..
                |          Set AComponent = ListComponents.Item("CheckOnHoles")
                |          ' .. and let's see what makes it true
                |          Set NupletsList = AComponent.Succeeds
                |          For i = 1 to AComponent.Succeeds.CountSucceed
                |            Set ANuplet = AComponent.Succeeds.SucceedItem(i)
                |            NupletSize = ANuplet.getTupleSize()
                |            ReDim anArray (NupletSize)
                |            ANuplet.getTuple(anArray)
                |            For j = LBound(anArray) to UBound(anArray)
                |              Set anElementArray = anArray(j) ' a hole
                | 
                |              ' .. some action on the element of the array
                |            Next
                |          Next
                |          
                | 
                |     Parameters:
                | 
                |         oSafeArray
                |             The collection of objects.

        :param tuple o_safe_array:
        :rtype: None
        """
        return self.expert_report_object.getTuple(o_safe_array)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_tuple'
        # # vba_code = """
        # # Public Function get_tuple(expert_report_object)
        # #     Dim oSafeArray (2)
        # #     expert_report_object.getTuple oSafeArray
        # #     get_tuple = oSafeArray
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_tuple_size(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func getTupleSize() As long
                | 
                |     Returns the size of the tuple concerned by this report
                |     object.
                | 
                |     Returns:
                |         Size of the tuple

        :rtype: int
        """
        return self.expert_report_object.getTupleSize()

    def __repr__(self):
        return f'ExpertReportObject(name="{ self.name }")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.dnb_mhi_interfaces.mhi_load_parameters import MHILoadParameters
from pycatia.system_interfaces.any_object import AnyObject


class MHISaveAccess(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MHISaveAccess
                | 
                | Interface representing a means to (1) Save data to the Hub (2) Retrieve certain
                | information on the loaded document
                | 
                | DNBIAMHISaveAccess is implemented on Document.
                | Applications and CAA partners should NOT implement this
                | interface.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.mhi_save_access = com_object

    def get_detailing_names(self, o_currently_loaded_detailing: str) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetDetailingNames(CATBSTR oCurrentlyLoadedDetailing) As
                | CATSafeArrayVariant
                | 
                |     Gets the name of the currently loaded detailing and the names of all the
                |     other detailings that exist for the loaded object.
                | 
                |     Parameters:
                | 
                |         oCurrentlyLoadedDetailing
                |             [inout] The name of the currently loaded detailing. This will be
                |             empty if no detailing was loaded 
                |         oListDetailingNames
                |             [out] The names of all the detailings that exist on the loaded
                |             object 
                | 
                |     Returns:
                |         S_OK if names returned OK S_FALSE if no detailings exist for the loaded
                |         object E_FAIL on error

        :param str o_currently_loaded_detailing:
        :rtype: tuple
        """
        return self.mhi_save_access.GetDetailingNames(o_currently_loaded_detailing)

    def get_load_parameters(self) -> MHILoadParameters:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetLoadParameters() As MHILoadParameters
                | 
                |     Gets the interface to the Load Parameters object that contains all the
                |     information pertaining to the load of the document
                | 
                |     Parameters:
                | 
                |         oLoadParameters
                |             [out] Interface to the Load Parameters object 
                | 
                |     Returns:
                |         S_OK if everything ran OK E_FAIL on error

        :rtype: MHILoadParameters
        """
        return MHILoadParameters(self.mhi_save_access.GetLoadParameters())

    def save_to_ppr_hub(self, i_detailing_name: str, i_overwrite_detailing: bool) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func SaveToPPRHub(CATBSTR iDetailingName,
                | boolean iOverwriteDetailing) As CATSafeArrayVariant
                | 
                |     This method saves a document from V5 to the PPR Hub Note: All
                |     Tools->Options settings will be respected during the save
                | 
                |     Parameters:
                | 
                |         iDetailingName
                |             [in] The name of the detailing to save Note: If the Tools->Options
                |             'Save Without Detailing' is checked ON, then NO detailing will be saved. This
                |             argument will be ignored and only the exposed data will be saved to the Hub
                |             
                |         iOverwriteDetailing
                |             [in] Option to indicate if a detailing with the same name needs to
                |             overwritten If this argument is FALSE and a detailing with the same name
                |             exists, then the save will be aborted (Neither detailing nor exposed data will
                |             be saved) 
                |         oListErrorMessages
                |             [out] List of error messages about invalid or incorrect arguments
                |             (for example Tools->Options settings conflicting with input arguments to
                |             method). Note: This will not contain the error messages that occured during the
                |             save (for e.g., failure to save an object/relation in the database)
                |
                |     Returns:
                |         S_OK if everything ran OK E_FAIL on error

        :param str i_detailing_name:
        :param bool i_overwrite_detailing:
        :rtype: tuple
        """
        return self.mhi_save_access.SaveToPPRHub(i_detailing_name, i_overwrite_detailing)

    def __repr__(self):
        return f'MHISaveAccess(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from pycatia.system_interfaces.any_object import AnyObject


class Reference(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Reference
                |
                | Represents an object pointing to another object.
                | This other object can be either a wireframe GeometricElement object such as a
                | plane or a line, or a boundary representation object such as a face, a vertex
                | or an edge. It may be, in particular, a Boundary object. References are created
                | using appropriate methods for parts. They are then passed to an object to
                | enable associativity with the referenced object.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.reference = com_object

    @property
    def display_name(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property DisplayName() As CATBSTR (Read Only)
                | 
                |     Returns the name of the referenced object. The name of the referenced
                |     object is either the name displayed in the specification tree for a
                |     GeometricElement object or a character string defining the reference for a
                |     boundary object.
                |
                |     Example:
                |         The following example returns in StrName the displayable name of
                |         reference FirstRef:
                |
                |          StrName = FirstRef.DisplayName

        :rtype: str
        """

        return self.reference.DisplayName

    def compose_with(self, i_reference: 'Reference') -> 'Reference':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func ComposeWith(Reference iReference) As Reference
                | 
                |     Composes a reference with another reference thus creating a new composite
                |     reference.
                |
                |     Parameters:
                |
                |         iReference
                |             The reference to be composed with the current
                |             reference.
                | 
                |             Example:
                |                 The following example returns in CompositeRef the reference
                |                 resulting from the composition of the FirstRef and SecondRef
                |                 references.
                | 
                |                  Dim CompositeRef As Reference
                |                  Set CompositeRef = FirstRef.ComposeWith(SecondRef)

        :param Reference i_reference:
        :rtype: Reference
        """
        return Reference(self.reference.ComposeWith(i_reference.com_object))

    def __repr__(self):
        return f'Reference(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_sch_platform_interfaces.sch_grr_comp import SchGRRComp
from pycatia.cat_sch_platform_interfaces.sch_list_of_objects import SchListOfObjects
from pycatia.system_interfaces.any_object import AnyObject


class SchCompGraphic(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SchCompGraphic
                | 
                | Manage the graphical representation of a schematic component.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sch_comp_graphic = com_object

    def activate(self, i_grr_name: str, i_db2_where_at: tuple, o_grr: SchGRRComp) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Activate(CATBSTR iGRRName,
                | CATSafeArrayVariant iDb2WhereAt,
                | SchGRRComp oGRR)
                | 
                |     To add a new image to an existing object. This new image is an instance of
                |     graphical representation with the input name.
                | 
                |     Parameters:
                | 
                |         iGRRName
                |             The name of the graphic representation 
                |         iDb2WhereAt
                |             The x-y coordinates of the image position. If NULL, the image will
                |             be positioned at the origin. 
                |         oGRR
                |             Pointer to the new graphical image of the component.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim strVar1 As String
                |          Dim dbVar2(2) As CATSafeArrayVariant
                |          Dim objArg3 As SchGRRComp
                |           ...
                |          objThisIntf.ActivatestrVar1,dbVar2,objArg3

        :param str i_grr_name:
        :param tuple i_db2_where_at:
        :param SchGRRComp o_grr:
        :rtype: tuple
        """
        return self.sch_comp_graphic.Activate(i_grr_name, i_db2_where_at, o_grr.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'activate'
        # # vba_code = """
        # # Public Function activate(sch_comp_graphic)
        # #     Dim iGRRName (2)
        # #     sch_comp_graphic.Activate iGRRName
        # #     activate = iGRRName
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def add_graphical_representation(self, i_grr_to_add: SchGRRComp) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub AddGraphicalRepresentation(SchGRRComp iGRRToAdd)
                | 
                |     Add a graphical representation to a component.
                | 
                |     Parameters:
                | 
                |         iGRRToAdd
                |             The graphical representation to be added to the component.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchGRRComp
                |           ...
                |          objThisIntf.AddGraphicalRepresentationobjArg1

        :param SchGRRComp i_grr_to_add:
        :rtype: None
        """
        return self.sch_comp_graphic.AddGraphicalRepresentation(i_grr_to_add.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_graphical_representation'
        # # vba_code = """
        # # Public Function add_graphical_representation(sch_comp_graphic)
        # #     Dim iGRRToAdd (2)
        # #     sch_comp_graphic.AddGraphicalRepresentation iGRRToAdd
        # #     add_graphical_representation = iGRRToAdd
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def deactivate(self, i_grr: SchGRRComp) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Deactivate(SchGRRComp iGRR)
                | 
                |     To remove an image to an existing object.
                | 
                |     Parameters:
                | 
                |         iGRR
                |             The graphical image to be removed from the component.
                |             
                |         iDb2WhereAt
                |             The x-y coordinates of the image position. If NULL, the image will
                |             be positioned at the origin. 
                | 
                |     Example:
                | 
                |           
                | 
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchGRRComp
                |           ...
                |          objThisIntf.DeactivateobjArg1

        :param SchGRRComp i_grr:
        :rtype: None
        """
        return self.sch_comp_graphic.Deactivate(i_grr.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'deactivate'
        # # vba_code = """
        # # Public Function deactivate(sch_comp_graphic)
        # #     Dim iGRR (2)
        # #     sch_comp_graphic.Deactivate iGRR
        # #     deactivate = iGRR
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def list_graphical_images(self) -> SchListOfObjects:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ListGraphicalImages() As SchListOfObjects
                | 
                |     List all graphical images (instances of the rep) of a
                |     component.
                | 
                |     Parameters:
                | 
                |         oLGRR
                |             A list of graphical images (members are CATISchGRRComp interface
                |             pointers). 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchListOfObjects
                |           ...
                |          Set objArg1 = objThisIntf.ListGraphicalImages

        :rtype: SchListOfObjects
        """
        return SchListOfObjects(self.sch_comp_graphic.ListGraphicalImages())

    def list_graphical_representations(self) -> SchListOfObjects:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ListGraphicalRepresentations() As SchListOfObjects
                | 
                |     List all graphical representation of a component.
                | 
                |     Parameters:
                | 
                |         oLGRR
                |             A list of graphical representations (members are CATISchGRRComp
                |             interface pointers). 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchListOfObjects
                |           ...
                |          Set objArg1 = objThisIntf.ListGraphicalRepresentations

        :rtype: SchListOfObjects
        """
        return SchListOfObjects(self.sch_comp_graphic.ListGraphicalRepresentations())

    def remove_graphical_representation(self, i_grr_to_remove: SchGRRComp) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub RemoveGraphicalRepresentation(SchGRRComp iGRRToRemove)
                | 
                |     Remove a graphical representation from a component.
                | 
                |     Parameters:
                | 
                |         iGRRToRemove
                |             The graphical representation to be removed from the component.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchGRRComp
                |           ...
                |          objThisIntf.RemoveGraphicalRepresentationobjArg1

        :param SchGRRComp i_grr_to_remove:
        :rtype: None
        """
        return self.sch_comp_graphic.RemoveGraphicalRepresentation(i_grr_to_remove.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove_graphical_representation'
        # # vba_code = """
        # # Public Function remove_graphical_representation(sch_comp_graphic)
        # #     Dim iGRRToRemove (2)
        # #     sch_comp_graphic.RemoveGraphicalRepresentation iGRRToRemove
        # #     remove_graphical_representation = iGRRToRemove
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def switch(self, i_grr: SchGRRComp, i_grr_name: str, o_grr: SchGRRComp) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Switch(SchGRRComp iGRR,
                | CATBSTR iGRRName,
                | SchGRRComp oGRR)
                | 
                |     Replace the input image object with an image of the graphical
                |     representation with the input name.
                | 
                |     Parameters:
                | 
                |         iGRR
                |             Pointer to the component graphical image to be switched.
                |             
                |         oGRR
                |             Pointer to the new graphical image of the component.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim objArg1 As SchGRRComp
                |          Dim strVar2 As String
                |          Dim objArg3 As SchGRRComp
                |           ...
                |          objThisIntf.SwitchobjArg1,strVar2,objArg3

        :param SchGRRComp i_grr:
        :param str i_grr_name:
        :param SchGRRComp o_grr:
        :rtype: None
        """
        return self.sch_comp_graphic.Switch(i_grr.com_object, i_grr_name, o_grr.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'switch'
        # # vba_code = """
        # # Public Function switch(sch_comp_graphic)
        # #     Dim iGRR (2)
        # #     sch_comp_graphic.Switch iGRR
        # #     switch = iGRR
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def switch_all(self, i_grr_name: str) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SwitchAll(CATBSTR iGRRName)
                | 
                |     Replace all occurances of the images of this component with those of the
                |     graphical representation with the input name.
                | 
                |     Parameters:
                | 
                |         iGRRName
                |             The name of the graphical representation 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchCompGraphic
                |          Dim strVar1 As String
                |           ...
                |          objThisIntf.SwitchAllstrVar1

        :param str i_grr_name:
        :rtype: None
        """
        return self.sch_comp_graphic.SwitchAll(i_grr_name)

    def __repr__(self):
        return f'SchCompGraphic(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.drafting_2dL_interfaces.layout_2d_sheet import Layout2DSheet
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class Layout2DSheets(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     Layout2DSheets
                | 
                | A collection of all the Layout sheets 2DL currently managed by the
                | LayoutRoot.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Layout2DSheet)
        self.layout_2d_sheets = com_object

    @property
    def active_sheet(self) -> Layout2DSheet:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property ActiveSheet() As Layout2DSheet (Read Only)
                | 
                |     Returns the active Layout sheet of the Layout document.
                | 
                |     Example:
                |         The following example shows how to get the active sheet and retrieved
                |         in MySheet in the Layout sheet collection of the layout root of Part supposed
                |         to be in the active document
                | 
                |          Dim MyLayoutRoot As Layout2DRoot
                |          Set MyLayoutRoot = CATIA.Documents.Part.GetItem("CATLayoutRoot")
                |          Dim MySheet As Layout2DSheet
                |          Set MySheet =  MyLayoutRoot.Sheets.ActiveSheet

        :rtype: Layout2DSheet
        """

        return Layout2DSheet(self.layout_2d_sheets.ActiveSheet)

    def add(self, i_layout_sheet_name: str) -> Layout2DSheet:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Add(CATBSTR iLayoutSheetName) As Layout2DSheet
                | 
                |     Creates a Layout sheet and adds it to the Layout2DSheets collection. This
                |     Layout sheet becomes the active one.
                | 
                |     Parameters:
                | 
                |         iLayoutSheetName
                |             The name to assign to the created Layout2DSheet object
                |             
                | 
                |     Returns:
                |         The created Layout sheet 
                |     Example:
                |         The following example creates a Layout sheet named FirstSheet and
                |         retrieved in MySheet in the Layout sheet collection of the layout root of Part
                |         supposed to be in the active document
                | 
                |          Dim MyLayoutRoot As Layout2DRoot
                |          Set MyLayoutRoot = CATIA.Documents.Part.GetItem("CATLayoutRoot")
                |          Dim MySheet As Layout2DSheet
                |          Set MySheet = MyLayoutRoot.Sheets.Add("FirstSheet").

        :param str i_layout_sheet_name:
        :rtype: Layout2DSheet
        """
        return Layout2DSheet(self.layout_2d_sheets.Add(i_layout_sheet_name))

    def add_detail(self, i_layout_sheet_name: str) -> Layout2DSheet:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddDetail(CATBSTR iLayoutSheetName) As Layout2DSheet
                | 
                |     Creates a detail Layout sheet 2DL and adds it to the LayoutSheets2DL
                |     collection. This detail Layout sheet becomes the active
                |     one.
                | 
                |     Parameters:
                | 
                |         iLayoutSheetName
                |             The name to assign to the created detail LayoutSheet object
                |             
                | 
                |     Returns:
                |         The created layout sheet 
                |     Example:
                |         The following example creates a detail Layout sheet named FirstSheet
                |         and retrieved in MySheet in the Layout sheet collection of the layout root of
                |         Part supposed to be in the active document
                | 
                |          Dim MyLayoutRoot As Layout2DRoot
                |          Set MyLayoutRoot = CATIA.Documents.Part.GetItem("CATLayoutRoot")
                |          Dim MySheet As Layout2DSheet
                |          Set MySheet = MyLayoutRoot.Sheets.Add("FirstSheet")

        :param str i_layout_sheet_name:
        :rtype: Layout2DSheet
        """
        return Layout2DSheet(self.layout_2d_sheets.AddDetail(i_layout_sheet_name))

    def item(self, i_index: cat_variant) -> Layout2DSheet:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Item(CATVariant iIndex) As Layout2DSheet
                | 
                |     Returns a Layout sheet using its index or its name from the Layout2DSheets
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Layout sheet to retrieve from the
                |             collection of Layout sheets. As a numerics, this index is the rank of the
                |             Layout sheet in the collection. The index of the first Layout sheet in the
                |             collection is 1, and the index of the last Layout sheet is Count. As a string,
                |             it is the name you assigned to the Layout sheet using the
                |             
                | 
                |         AnyObject.Name property or when creating it using the Add method.
                |         
                |     Returns:
                |         The retrieved Layout sheet 
                |     Example:
                |         This example retrieves in ThisLayoutSheet the third Layout sheet, and
                |         in ThatLayoutSheet the Layout sheet named MySheet in the Layout sheet
                |         collection of the layout root of Part supposed to be in the active
                |         document.
                | 
                |          Dim ThisLayoutRoot As Layout2DRoot
                |          Set ThisLayoutRoot = CATIA.ActiveDocument.Part.GetItem("CATlayoutRoot")
                |          Dim ThisLayoutSheet As Layout2DSheet
                |          Set ThisLayoutSheet = ThisLayoutRoot.Sheets.Item(3)
                |          Dim ThatLayoutSheet As Layout2DSheet
                |          Set ThatLayoutSheet = ThisLayoutRoot.Sheets.Item("MySheet")

        :param cat_variant i_index:
        :rtype: Layout2DSheet
        """
        return Layout2DSheet(self.layout_2d_sheets.Item(i_index))

    def remove(self, i_index: cat_variant) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Remove(CATVariant iIndex)
                | 
                |     Removes a Layout2Dsheet from the Layout2DSheets
                |     collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the Layout sheet to remove from the
                |             collection of Layout sheets. As a numerics, this index is the rank of the
                |             Layout sheet in the collection. The index of the first Layout sheet in the
                |             collection is 1, and the index of the last Layout sheet is Count. As a string,
                |             it is the name you assigned to the Layout sheet using the
                |             
                | 
                |         AnyObject.Name property or when creating it using the Add method.
                |         
                |     Example:
                |         The following example removes the second Layout sheet and the Layout
                |         sheet named SheetToBeRemoved in the Layout sheet collection of the layout root
                |         of Part supposed to be in the active document.
                | 
                |          Dim ThisLayoutRoot As Layout2DRoot
                |          Set ThisLayoutRoot = CATIA.ActiveDocument.Part.GetItem("CATlayoutRoot")
                |         
ThisLayoutRoot.Layout2DSheets.Remove("SheetToBeRemoved")

        :param cat_variant i_index:
        :rtype: None
        """
        return self.layout_2d_sheets.Remove(i_index)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'remove'
        # # vba_code = """
        # # Public Function remove(layout2_d_sheets)
        # #     Dim iIndex (2)
        # #     layout2_d_sheets.Remove iIndex
        # #     remove = iIndex
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'Layout2DSheets(name="{self.name}")'
